'use strict';

var module$1 = require('module');
var eff = require('@eslint-react/eff');
var tsPattern = require('ts-pattern');
var z = require('@zod/mini');
var kit = require('@eslint-react/kit');
var fastEquals = require('fast-equals');
var memoize = require('micro-memoize');

var _documentCurrentScript = typeof document !== 'undefined' ? document.currentScript : null;
function _interopDefault (e) { return e && e.__esModule ? e : { default: e }; }

function _interopNamespace(e) {
  if (e && e.__esModule) return e;
  var n = Object.create(null);
  if (e) {
    Object.keys(e).forEach(function (k) {
      if (k !== 'default') {
        var d = Object.getOwnPropertyDescriptor(e, k);
        Object.defineProperty(n, k, d.get ? d : {
          enumerable: true,
          get: function () { return e[k]; }
        });
      }
    });
  }
  n.default = e;
  return Object.freeze(n);
}

var module__default = /*#__PURE__*/_interopDefault(module$1);
var z__namespace = /*#__PURE__*/_interopNamespace(z);
var memoize__default = /*#__PURE__*/_interopDefault(memoize);

// src/constants.ts
var NPM_SCOPE = "@eslint-react";
var GITHUB_URL = "https://github.com/Rel1cx/eslint-react";
var WEBSITE_URL = "https://eslint-react.xyz";

// src/get-doc-url.ts
var getDocsUrl = (pluginName) => (ruleName) => {
  if (pluginName === "x") {
    return `${WEBSITE_URL}/docs/rules/${ruleName}`;
  }
  return `${WEBSITE_URL}/docs/rules/${pluginName}-${ruleName}`;
};

// src/get-id.ts
var id = 0n;
var getId = () => (id++).toString();
var _require = module__default.default.createRequire((typeof document === 'undefined' ? require('u' + 'rl').pathToFileURL(__filename).href : (_documentCurrentScript && _documentCurrentScript.tagName.toUpperCase() === 'SCRIPT' && _documentCurrentScript.src || new URL('index.js', document.baseURI).href)));
function getReactVersion(fallback = "19.0.0") {
  try {
    return tsPattern.match(_require("react")).with({ version: tsPattern.P.select(tsPattern.P.string) }, eff.identity).otherwise(() => fallback);
  } catch {
    return fallback;
  }
}
var CustomComponentPropSchema = z__namespace.object({
  /**
   * The name of the prop in the user-defined component.
   * @example
   * "to"
   */
  name: z__namespace.string(),
  /**
   * The name of the prop in the host component.
   * @example
   * "href"
   */
  as: z__namespace.optional(z__namespace.string()),
  /**
   * Whether the prop is controlled or not in the user-defined component.
   * @internal
   * @example
   * `true`
   */
  controlled: z__namespace.optional(z__namespace.boolean()),
  /**
   * The default value of the prop in the user-defined component.
   * @example
   * `"/"`
   */
  defaultValue: z__namespace.optional(z__namespace.string())
});
var CustomComponentSchema = z__namespace.object({
  /**
   * The name of the user-defined component.
   * @example
   * "Link"
   */
  name: z__namespace.string(),
  /**
   * The name of the host component that the user-defined component represents.
   * @example
   * "a"
   */
  as: z__namespace.optional(z__namespace.string()),
  /**
   * Attributes mapping between the user-defined component and the host component.
   * @example
   * `Link` component has a `to` attribute that represents the `href` attribute in the built-in `a` element with a default value of `"/"`.
   */
  attributes: z__namespace.optional(z__namespace.array(CustomComponentPropSchema)),
  /**
   * The ESQuery selector to select the component precisely.
   * @internal
   * @example
   * `JSXElement:has(JSXAttribute[name.name='component'][value.value='a'])`
   */
  selector: z__namespace.optional(z__namespace.string())
});
var CustomHooksSchema = z__namespace.object({
  use: z__namespace.optional(z__namespace.array(z__namespace.string())),
  useActionState: z__namespace.optional(z__namespace.array(z__namespace.string())),
  useCallback: z__namespace.optional(z__namespace.array(z__namespace.string())),
  useContext: z__namespace.optional(z__namespace.array(z__namespace.string())),
  useDebugValue: z__namespace.optional(z__namespace.array(z__namespace.string())),
  useDeferredValue: z__namespace.optional(z__namespace.array(z__namespace.string())),
  useEffect: z__namespace.optional(z__namespace.array(z__namespace.string())),
  useFormStatus: z__namespace.optional(z__namespace.array(z__namespace.string())),
  useId: z__namespace.optional(z__namespace.array(z__namespace.string())),
  useImperativeHandle: z__namespace.optional(z__namespace.array(z__namespace.string())),
  useInsertionEffect: z__namespace.optional(z__namespace.array(z__namespace.string())),
  useLayoutEffect: z__namespace.optional(z__namespace.array(z__namespace.string())),
  useMemo: z__namespace.optional(z__namespace.array(z__namespace.string())),
  useOptimistic: z__namespace.optional(z__namespace.array(z__namespace.string())),
  useReducer: z__namespace.optional(z__namespace.array(z__namespace.string())),
  useRef: z__namespace.optional(z__namespace.array(z__namespace.string())),
  useState: z__namespace.optional(z__namespace.array(z__namespace.string())),
  useSyncExternalStore: z__namespace.optional(z__namespace.array(z__namespace.string())),
  useTransition: z__namespace.optional(z__namespace.array(z__namespace.string()))
});
var ESLintReactSettingsSchema = z__namespace.object({
  /**
   * The source where React is imported from.
   * @description This allows to specify a custom import location for React when not using the official distribution.
   * @default `"react"`
   * @example `"@pika/react"`
   */
  importSource: z__namespace.optional(z__namespace.string()),
  /**
   * The identifier that's used for JSX Element creation.
   * @default `"createElement"`
   * @deprecated
   */
  jsxPragma: z__namespace.optional(z__namespace.string()),
  /**
   * The identifier that's used for JSX fragment elements.
   * @description This should not be a member expression (i.e. use "Fragment" instead of "React.Fragment").
   * @default `"Fragment"`
   * @deprecated
   */
  jsxPragmaFrag: z__namespace.optional(z__namespace.string()),
  /**
   * The name of the prop that is used for polymorphic components.
   * @description This is used to determine the type of the component.
   * @example `"as"`
   */
  polymorphicPropName: z__namespace.optional(z__namespace.string()),
  /**
   * @internal
   */
  strict: z__namespace.optional(z__namespace.boolean()),
  /**
   * Check both the shape and the import to determine if an API is from React.
   * @description This can prevent false positives when using a irrelevant third-party library that has similar APIs to React.
   * @default `true`
   */
  skipImportCheck: z__namespace.optional(z__namespace.boolean()),
  /**
   * React version to use, "detect" means auto detect React version from the project's dependencies.
   * If `importSource` is specified, an equivalent version of React should be provided here.
   * @example `"18.3.1"`
   * @default `"detect"`
   */
  version: z__namespace.optional(z__namespace.string()),
  /**
   * A object to define additional hooks that are equivalent to the built-in React Hooks.
   * @description ESLint React will recognize these aliases as equivalent to the built-in hooks in all its rules.
   * @example `{ useEffect: ["useIsomorphicLayoutEffect"] }`
   */
  additionalHooks: z__namespace.optional(CustomHooksSchema),
  /**
   * An array of user-defined components
   * @description This is used to inform the ESLint React plugins how to treat these components during checks.
   * @example `[{ name: "Link", as: "a", attributes: [{ name: "to", as: "href" }, { name: "rel", defaultValue: "noopener noreferrer" }] }]`
   */
  additionalComponents: z__namespace.optional(z__namespace.array(CustomComponentSchema))
});
var ESLintSettingsSchema = z__namespace.optional(
  z__namespace.object({
    "react-x": z__namespace.optional(ESLintReactSettingsSchema),
    /** @deprecated Use `react-x` instead */
    reactOptions: z__namespace.optional(ESLintReactSettingsSchema)
  }),
  {}
);
var DEFAULT_ESLINT_REACT_SETTINGS = {
  ...z__namespace.parse(ESLintReactSettingsSchema, {}),
  version: "detect",
  importSource: "react",
  strict: false,
  skipImportCheck: true,
  polymorphicPropName: "as",
  additionalHooks: {
    useEffect: ["useIsomorphicLayoutEffect"],
    useLayoutEffect: ["useIsomorphicLayoutEffect"]
  }
};
function unsafeDecodeSettings(data) {
  return data?.["react-x"] ?? {};
}
var decodeSettings = memoize__default.default((data) => {
  return {
    ...DEFAULT_ESLINT_REACT_SETTINGS,
    ...z__namespace.parse(ESLintSettingsSchema, data)?.["react-x"] ?? {}
  };
}, { isEqual: (a, b) => a === b });
var toNormalizedSettings = memoize__default.default(
  ({
    additionalComponents = [],
    additionalHooks = {},
    importSource = "react",
    polymorphicPropName = "as",
    skipImportCheck = true,
    // strict = false,
    version,
    ...rest
  }) => {
    return {
      ...rest,
      additionalComponents: additionalComponents.map(({
        name,
        as = name,
        attributes = [],
        ...rest2
      }) => ({
        ...rest2,
        name,
        as,
        attributes: attributes.map(({ name: name2, as: as2 = name2, ...rest3 }) => ({
          ...rest3,
          name: name2,
          as: as2
        })),
        re: kit.RE.toRegExp(name)
      })),
      additionalHooks,
      importSource,
      polymorphicPropName,
      skipImportCheck,
      version: tsPattern.match(version).with(tsPattern.P.union(tsPattern.P.nullish, "", "detect"), () => getReactVersion("19.1.0")).otherwise(eff.identity)
    };
  },
  { isEqual: fastEquals.shallowEqual }
);
function getSettingsFromContext(context) {
  return toNormalizedSettings(decodeSettings(context.settings));
}
var defineSettings = eff.identity;

exports.CustomComponentPropSchema = CustomComponentPropSchema;
exports.CustomComponentSchema = CustomComponentSchema;
exports.CustomHooksSchema = CustomHooksSchema;
exports.DEFAULT_ESLINT_REACT_SETTINGS = DEFAULT_ESLINT_REACT_SETTINGS;
exports.ESLintReactSettingsSchema = ESLintReactSettingsSchema;
exports.ESLintSettingsSchema = ESLintSettingsSchema;
exports.GITHUB_URL = GITHUB_URL;
exports.NPM_SCOPE = NPM_SCOPE;
exports.WEBSITE_URL = WEBSITE_URL;
exports.decodeSettings = decodeSettings;
exports.defineSettings = defineSettings;
exports.getDocsUrl = getDocsUrl;
exports.getId = getId;
exports.getReactVersion = getReactVersion;
exports.getSettingsFromContext = getSettingsFromContext;
exports.toNormalizedSettings = toNormalizedSettings;
exports.unsafeDecodeSettings = unsafeDecodeSettings;
